require 'nexaas/cipher/salt'

module Nexaas
  module Cipher
    # CrypterBase: abstract class to support crypter classes
    class CrypterBase
      attr_reader :salt

      def initialize(*args)
        @salt = Salt.new(*args)
      end

      def encrypt(code, **options)
        guarded_crypt(code, :do_encrypt, **options)
      end

      def decrypt(code, **options)
        guarded_crypt(code, :do_decrypt, **options)
      end

      def salt?
        salt.salt?
      end

      protected

      def guarded_crypt(code, method, **options)
        return if code.nil? || code.empty?
        return code unless salt.salt?
        start_encrypting(code, method, **options)
      end

      # base64 :reek:ControlParameter
      # base64 :reek:BooleanParameter
      #        :reek:LongParameterList
      def start_encrypting(code, method, base64: false, **options)
        if base64
          b64_encrypt(code, method, **options)
        else
          do_crypt(code, method, **options)
        end
      end

      def b64_encrypt(code, method, **options)
        Base64.
          encode64(do_crypt(Base64.decode64(code), method, **options)).
          rstrip
      end

      def do_crypt(code, method, **options)
        case method
        when :do_encrypt
          do_encrypt(code, **options)
        when :do_decrypt
          do_decrypt(code, **options)
        else
          raise ArgumentError(method)
        end
      end
    end
  end
end
